package com.sun.jmx.mbeanserver;

import javax.management.Attribute;
import javax.management.ObjectName;
import javax.management.MBeanServerConnection;
import javax.management.NotCompliantMBeanException;
import java.lang.reflect.Method;
import java.util.Map;

public class MXBeanProxy
{
    private final Map<Method, MXBeanProxy.Handler> handlerMap;
    
    public MXBeanProxy(Class<?> clazz) {
        this.handlerMap = Util.newMap();
        if (clazz == null) {
            throw new IllegalArgumentException("Null parameter");
        }
        MBeanAnalyzer<ConvertingMethod> analyzer;
        try {
            analyzer = MXBeanIntrospector.getInstance().getAnalyzer(clazz);
        }
        catch (NotCompliantMBeanException cause) {
            throw new IllegalArgumentException(cause);
        }
        analyzer.visit(new MXBeanProxy.Visitor());
        return;
    }
    
    public Object invoke(MBeanServerConnection mBeanServerConnection, ObjectName objectName, Method method, Object[] array) throws Throwable {
        MXBeanProxy.Handler handler = (MXBeanProxy.Handler)this.handlerMap.get(method);
        ConvertingMethod convertingMethod = handler.getConvertingMethod();
        MXBeanLookup lookup = MXBeanLookup.lookupFor(mBeanServerConnection);
        MXBeanLookup lookup2 = MXBeanLookup.getLookup();
        try {
            MXBeanLookup.setLookup(lookup);
            return convertingMethod.fromOpenReturnValue(lookup, handler.invoke(mBeanServerConnection, objectName, convertingMethod.toOpenParameters(lookup, array)));
        }
        finally {
            MXBeanLookup.setLookup(lookup2);
        }
    }
    
    private class Visitor implements MBeanAnalyzer.MBeanVisitor<ConvertingMethod>
    {
        @Override
        public void visitAttribute(String s, ConvertingMethod convertingMethod, ConvertingMethod convertingMethod2) {
            if (convertingMethod != null) {
                convertingMethod.checkCallToOpen();
                MXBeanProxy.this.handlerMap.put(convertingMethod.getMethod(), new MXBeanProxy.GetHandler(s, convertingMethod));
            }
            if (convertingMethod2 != null) {
                MXBeanProxy.this.handlerMap.put(convertingMethod2.getMethod(), new MXBeanProxy.SetHandler(s, convertingMethod2));
            }
        }
        
        @Override
        public void visitOperation(String s, ConvertingMethod convertingMethod) {
            convertingMethod.checkCallToOpen();
            MXBeanProxy.this.handlerMap.put(convertingMethod.getMethod(), new MXBeanProxy.InvokeHandler(s, convertingMethod.getOpenSignature(), convertingMethod));
        }
    }
    
    private abstract static class Handler
    {
        private final String name;
        private final ConvertingMethod convertingMethod;
        
        Handler(String name, ConvertingMethod convertingMethod) {
            this.name = name;
            this.convertingMethod = convertingMethod;
        }
        
        String getName() {
            return this.name;
        }
        
        ConvertingMethod getConvertingMethod() {
            return this.convertingMethod;
        }
        
        abstract Object invoke(MBeanServerConnection p0, ObjectName p1, Object[] p2) throws Exception;
    }
    
    private static class GetHandler extends MXBeanProxy.Handler
    {
        GetHandler(String s, ConvertingMethod convertingMethod) {
            super(s, convertingMethod);
        }
        
        @Override
        Object invoke(MBeanServerConnection mBeanServerConnection, ObjectName objectName, Object[] array) throws Exception {
            assert array.length == 0;
            return mBeanServerConnection.getAttribute(objectName, this.getName());
        }
    }
    
    private static class SetHandler extends MXBeanProxy.Handler
    {
        SetHandler(String s, ConvertingMethod convertingMethod) {
            super(s, convertingMethod);
        }
        
        @Override
        Object invoke(MBeanServerConnection mBeanServerConnection, ObjectName objectName, Object[] array) throws Exception {
            assert array.length == 1;
            mBeanServerConnection.setAttribute(objectName, new Attribute(this.getName(), array[0]));
            return null;
        }
    }
    
    private static class InvokeHandler extends MXBeanProxy.Handler
    {
        private final String[] signature;
        
        InvokeHandler(String s, String[] signature, ConvertingMethod convertingMethod) {
            super(s, convertingMethod);
            this.signature = signature;
        }
        
        @Override
        Object invoke(MBeanServerConnection mBeanServerConnection, ObjectName objectName, Object[] array) throws Exception {
            return mBeanServerConnection.invoke(objectName, this.getName(), array, this.signature);
        }
    }
}
